# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import os
import typing

import httpx
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.logging import LogConfig, Logger
from .environment import SmallestAIEnvironment

if typing.TYPE_CHECKING:
    from .atoms.client import AsyncAtomsClient, AtomsClient
    from .waves.client import AsyncWavesClient, WavesClient


class SmallestAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : SmallestAIEnvironment
        The environment to use for requests from the client. from .environment import SmallestAIEnvironment



        Defaults to SmallestAIEnvironment.PRODUCTION



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    logging : typing.Optional[typing.Union[LogConfig, Logger]]
        Configure logging for the SDK. Accepts a LogConfig dict with 'level' (debug/info/warn/error), 'logger' (custom logger implementation), and 'silent' (boolean, defaults to True) fields. You can also pass a pre-configured Logger instance.

    Examples
    --------
    from smallestai import SmallestAI

    client = SmallestAI(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        environment: SmallestAIEnvironment = SmallestAIEnvironment.PRODUCTION,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SMALLEST_AI_TOKEN"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is None:
            raise ApiError(
                body="The client must be instantiated be either passing in token or setting SMALLEST_AI_TOKEN"
            )
        self._client_wrapper = SyncClientWrapper(
            environment=environment,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
            logging=logging,
        )
        self._atoms: typing.Optional[AtomsClient] = None
        self._waves: typing.Optional[WavesClient] = None

    @property
    def atoms(self):
        if self._atoms is None:
            from .atoms.client import AtomsClient  # noqa: E402

            self._atoms = AtomsClient(client_wrapper=self._client_wrapper)
        return self._atoms

    @property
    def waves(self):
        if self._waves is None:
            from .waves.client import WavesClient  # noqa: E402

            self._waves = WavesClient(client_wrapper=self._client_wrapper)
        return self._waves


class AsyncSmallestAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : SmallestAIEnvironment
        The environment to use for requests from the client. from .environment import SmallestAIEnvironment



        Defaults to SmallestAIEnvironment.PRODUCTION



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    logging : typing.Optional[typing.Union[LogConfig, Logger]]
        Configure logging for the SDK. Accepts a LogConfig dict with 'level' (debug/info/warn/error), 'logger' (custom logger implementation), and 'silent' (boolean, defaults to True) fields. You can also pass a pre-configured Logger instance.

    Examples
    --------
    from smallestai import AsyncSmallestAI

    client = AsyncSmallestAI(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        environment: SmallestAIEnvironment = SmallestAIEnvironment.PRODUCTION,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SMALLEST_AI_TOKEN"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is None:
            raise ApiError(
                body="The client must be instantiated be either passing in token or setting SMALLEST_AI_TOKEN"
            )
        self._client_wrapper = AsyncClientWrapper(
            environment=environment,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
            logging=logging,
        )
        self._atoms: typing.Optional[AsyncAtomsClient] = None
        self._waves: typing.Optional[AsyncWavesClient] = None

    @property
    def atoms(self):
        if self._atoms is None:
            from .atoms.client import AsyncAtomsClient  # noqa: E402

            self._atoms = AsyncAtomsClient(client_wrapper=self._client_wrapper)
        return self._atoms

    @property
    def waves(self):
        if self._waves is None:
            from .waves.client import AsyncWavesClient  # noqa: E402

            self._waves = AsyncWavesClient(client_wrapper=self._client_wrapper)
        return self._waves



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel
from .single_prompt_config_tools_item_variables_extraction_schema_item_type import (
    SinglePromptConfigToolsItemVariablesExtractionSchemaItemType,
)


class SinglePromptConfigToolsItemVariablesExtractionSchemaItem(UncheckedBaseModel):
    name: str = pydantic.Field()
    """
    Name of the variable to extract
    """

    description: str = pydantic.Field()
    """
    Description of what this variable represents
    """

    type: SinglePromptConfigToolsItemVariablesExtractionSchemaItemType = pydantic.Field()
    """
    Data type of the variable
    """

    values: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Required when type is 'enum'. List of possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel


class StreamingTtsConfig(UncheckedBaseModel):
    """
    Configuration object for streaming TTS synthesis
    """

    voice_id: str = pydantic.Field()
    """
    Voice identifier
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    Language code
    """

    sample_rate: typing.Optional[int] = pydantic.Field(default=None)
    """
    Audio sample rate in Hz
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Speech speed multiplier
    """

    consistency: typing.Optional[float] = pydantic.Field(default=None)
    """
    Voice consistency parameter
    """

    enhancement: typing.Optional[int] = pydantic.Field(default=None)
    """
    Audio enhancement level
    """

    similarity: typing.Optional[float] = pydantic.Field(default=None)
    """
    Voice similarity parameter
    """

    max_buffer_flush_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum buffer time before forcing output
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

PostAgentAgentIdWebhookSubscriptionsRequestEventTypesItem = typing.Union[
    typing.Literal["pre-conversation", "post-conversation"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ..types.single_prompt_config import SinglePromptConfig
from ..types.workflow_type import WorkflowType
from .raw_client import AsyncRawWorkflowsClient, RawWorkflowsClient
from .types.patch_workflow_id_request_workflow_graph import PatchWorkflowIdRequestWorkflowGraph
from .types.patch_workflow_id_response import PatchWorkflowIdResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WorkflowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWorkflowsClient
        """
        return self._raw_client

    def update_workflow_configuration(
        self,
        id: str,
        *,
        type: WorkflowType,
        workflow_graph: typing.Optional[PatchWorkflowIdRequestWorkflowGraph] = OMIT,
        single_prompt_config: typing.Optional[SinglePromptConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchWorkflowIdResponse:
        """
        Update workflow configuration for an agent. Currently available for single_prompt workflows only. For workflow_graph configurations, please use the UI.

        Parameters
        ----------
        id : str
            The ID of the workflow to update

        type : WorkflowType
            The type of workflow (single_prompt or workflow_graph)

        workflow_graph : typing.Optional[PatchWorkflowIdRequestWorkflowGraph]
            Workflow graph configuration. Required when type is workflow_graph. Exactly one of workflowGraph or singlePromptConfig must be provided.

        single_prompt_config : typing.Optional[SinglePromptConfig]
            Configuration for single_prompt workflow type. Required when type is single_prompt. Exactly one of workflowGraph or singlePromptConfig must be provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchWorkflowIdResponse
            Workflow updated successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.workflows.update_workflow_configuration(
            id="60d0fe4f5311236168a109ca",
            type="workflow_graph",
        )
        """
        _response = self._raw_client.update_workflow_configuration(
            id,
            type=type,
            workflow_graph=workflow_graph,
            single_prompt_config=single_prompt_config,
            request_options=request_options,
        )
        return _response.data


class AsyncWorkflowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWorkflowsClient
        """
        return self._raw_client

    async def update_workflow_configuration(
        self,
        id: str,
        *,
        type: WorkflowType,
        workflow_graph: typing.Optional[PatchWorkflowIdRequestWorkflowGraph] = OMIT,
        single_prompt_config: typing.Optional[SinglePromptConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchWorkflowIdResponse:
        """
        Update workflow configuration for an agent. Currently available for single_prompt workflows only. For workflow_graph configurations, please use the UI.

        Parameters
        ----------
        id : str
            The ID of the workflow to update

        type : WorkflowType
            The type of workflow (single_prompt or workflow_graph)

        workflow_graph : typing.Optional[PatchWorkflowIdRequestWorkflowGraph]
            Workflow graph configuration. Required when type is workflow_graph. Exactly one of workflowGraph or singlePromptConfig must be provided.

        single_prompt_config : typing.Optional[SinglePromptConfig]
            Configuration for single_prompt workflow type. Required when type is single_prompt. Exactly one of workflowGraph or singlePromptConfig must be provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchWorkflowIdResponse
            Workflow updated successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.workflows.update_workflow_configuration(
                id="60d0fe4f5311236168a109ca",
                type="workflow_graph",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_workflow_configuration(
            id,
            type=type,
            workflow_graph=workflow_graph,
            single_prompt_config=single_prompt_config,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ... import core
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawKnowledgeBaseClient, RawKnowledgeBaseClient
from .types.delete_knowledgebase_id_response import DeleteKnowledgebaseIdResponse
from .types.delete_knowledgebase_knowledge_base_id_items_knowledge_base_item_id_response import (
    DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,
)
from .types.get_knowledgebase_id_items_response import GetKnowledgebaseIdItemsResponse
from .types.get_knowledgebase_id_response import GetKnowledgebaseIdResponse
from .types.get_knowledgebase_response import GetKnowledgebaseResponse
from .types.post_knowledgebase_id_items_upload_media_response import PostKnowledgebaseIdItemsUploadMediaResponse
from .types.post_knowledgebase_response import PostKnowledgebaseResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class KnowledgeBaseClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawKnowledgeBaseClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawKnowledgeBaseClient
        """
        return self._raw_client

    def get_all_knowledge_bases(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseResponse:
        """
        Get all knowledge bases

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseResponse
            A list of knowledge bases

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.get_all_knowledge_bases()
        """
        _response = self._raw_client.get_all_knowledge_bases(request_options=request_options)
        return _response.data

    def create_a_knowledge_base(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostKnowledgebaseResponse:
        """
        Create a knowledge base

        Parameters
        ----------
        name : str

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostKnowledgebaseResponse
            Knowledge base created successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.create_a_knowledge_base(
            name="name",
        )
        """
        _response = self._raw_client.create_a_knowledge_base(
            name=name, description=description, request_options=request_options
        )
        return _response.data

    def get_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseIdResponse:
        """
        Get a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseIdResponse
            A knowledge base

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.get_a_knowledge_base(
            id="id",
        )
        """
        _response = self._raw_client.get_a_knowledge_base(id, request_options=request_options)
        return _response.data

    def delete_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteKnowledgebaseIdResponse:
        """
        Delete a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteKnowledgebaseIdResponse
            Knowledge base deleted successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.delete_a_knowledge_base(
            id="id",
        )
        """
        _response = self._raw_client.delete_a_knowledge_base(id, request_options=request_options)
        return _response.data

    def get_all_knowledge_base_items(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseIdItemsResponse:
        """
        Get all knowledge base items

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseIdItemsResponse
            A list of knowledge base items

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.get_all_knowledge_base_items(
            id="id",
        )
        """
        _response = self._raw_client.get_all_knowledge_base_items(id, request_options=request_options)
        return _response.data

    def delete_a_knowledge_base_item(
        self,
        knowledge_base_id: str,
        knowledge_base_item_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse:
        """
        Delete a knowledge base item

        Parameters
        ----------
        knowledge_base_id : str
            The ID of the knowledge base

        knowledge_base_item_id : str
            The ID of the knowledge base item

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse
            Knowledge base item deleted successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.delete_a_knowledge_base_item(
            knowledge_base_id="knowledgeBaseId",
            knowledge_base_item_id="knowledgeBaseItemId",
        )
        """
        _response = self._raw_client.delete_a_knowledge_base_item(
            knowledge_base_id, knowledge_base_item_id, request_options=request_options
        )
        return _response.data

    def upload_a_pdf_file_to_a_knowledge_base(
        self, id: str, *, media: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PostKnowledgebaseIdItemsUploadMediaResponse:
        """
        Upload a PDF file to a knowledge base. Only PDF files are accepted.

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        media : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostKnowledgebaseIdItemsUploadMediaResponse
            Knowledge base item created successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.knowledge_base.upload_a_pdf_file_to_a_knowledge_base(
            id="id",
        )
        """
        _response = self._raw_client.upload_a_pdf_file_to_a_knowledge_base(
            id, media=media, request_options=request_options
        )
        return _response.data


class AsyncKnowledgeBaseClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawKnowledgeBaseClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawKnowledgeBaseClient
        """
        return self._raw_client

    async def get_all_knowledge_bases(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseResponse:
        """
        Get all knowledge bases

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseResponse
            A list of knowledge bases

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.get_all_knowledge_bases()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all_knowledge_bases(request_options=request_options)
        return _response.data

    async def create_a_knowledge_base(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostKnowledgebaseResponse:
        """
        Create a knowledge base

        Parameters
        ----------
        name : str

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostKnowledgebaseResponse
            Knowledge base created successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.create_a_knowledge_base(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_a_knowledge_base(
            name=name, description=description, request_options=request_options
        )
        return _response.data

    async def get_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseIdResponse:
        """
        Get a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseIdResponse
            A knowledge base

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.get_a_knowledge_base(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_a_knowledge_base(id, request_options=request_options)
        return _response.data

    async def delete_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteKnowledgebaseIdResponse:
        """
        Delete a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteKnowledgebaseIdResponse
            Knowledge base deleted successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.delete_a_knowledge_base(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_a_knowledge_base(id, request_options=request_options)
        return _response.data

    async def get_all_knowledge_base_items(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetKnowledgebaseIdItemsResponse:
        """
        Get all knowledge base items

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgebaseIdItemsResponse
            A list of knowledge base items

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.get_all_knowledge_base_items(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all_knowledge_base_items(id, request_options=request_options)
        return _response.data

    async def delete_a_knowledge_base_item(
        self,
        knowledge_base_id: str,
        knowledge_base_item_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse:
        """
        Delete a knowledge base item

        Parameters
        ----------
        knowledge_base_id : str
            The ID of the knowledge base

        knowledge_base_item_id : str
            The ID of the knowledge base item

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse
            Knowledge base item deleted successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.delete_a_knowledge_base_item(
                knowledge_base_id="knowledgeBaseId",
                knowledge_base_item_id="knowledgeBaseItemId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_a_knowledge_base_item(
            knowledge_base_id, knowledge_base_item_id, request_options=request_options
        )
        return _response.data

    async def upload_a_pdf_file_to_a_knowledge_base(
        self, id: str, *, media: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PostKnowledgebaseIdItemsUploadMediaResponse:
        """
        Upload a PDF file to a knowledge base. Only PDF files are accepted.

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        media : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostKnowledgebaseIdItemsUploadMediaResponse
            Knowledge base item created successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.knowledge_base.upload_a_pdf_file_to_a_knowledge_base(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_a_pdf_file_to_a_knowledge_base(
            id, media=media, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_agent_request_synthesizer_voice_config_gender import CreateAgentRequestSynthesizerVoiceConfigGender
from .create_agent_request_synthesizer_voice_config_one import CreateAgentRequestSynthesizerVoiceConfigOne

CreateAgentRequestSynthesizerVoiceConfig = typing.Union[
    CreateAgentRequestSynthesizerVoiceConfigGender, CreateAgentRequestSynthesizerVoiceConfigOne
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.unchecked_base_model import UncheckedBaseModel


class GetCampaignIdResponseDataMetrics(UncheckedBaseModel):
    """
    Campaign performance metrics
    """

    total_participants: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of contacts in the campaign audience
    """

    contacts_called: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of unique contacts where call was attempted (statuses IN_PROGRESS, COMPLETED, FAILED, NO_ANSWER)
    """

    contacts_connected: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of unique contacts who answered and had a conversation (status COMPLETED)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any, Dict, List


class ContextManager:
    def __init__(self):
        self._messages: List[Dict[str, Any]] = []

    @property
    def messages(self) -> List[Dict[str, Any]]:
        return self._messages

    def add_message(self, message_dict: Dict[str, Any]):
        self._messages.append(message_dict)

    def add_messages(self, messages: List[Dict[str, Any]]):
        self._messages.extend(messages)



"""
OutputAgentNode - Base class for agents that produce user-facing output.

Features:
- Automatically handles CONTROL_INTERRUPT
- Cancels streaming on interrupt
- Emits LLM_RESPONSE_START/CHUNK/END events
- Subclass implements generate_response()
"""

from abc import abstractmethod
from typing import Any, AsyncIterator, Dict

from loguru import logger

from smallestai.atoms.agent.context import ContextManager
from smallestai.atoms.agent.events import (
    OutputAgentSettings,
    SDKAgentErrorEvent,
    SDKAgentLLMResponseChunkEvent,
    SDKAgentLLMResponseEndEvent,
    SDKAgentLLMResponseStartEvent,
    SDKAgentSpeakEvent,
    SDKAgentTranscriptUpdateEvent,
    SDKEvent,
    SDKSystemInitEvent,
    SDKSystemLLMRequestEvent,
    SDKSystemUpdateOutputAgentSettingsEvent,
)
from smallestai.atoms.agent.nodes.base import Node
from smallestai.atoms.agent.task_manager import TaskManager


class OutputAgentNode(Node):
    """
    Base class for agents that produce user-facing output.

    This node type:
    - Automatically handles CONTROL_INTERRUPT events
    - Cancels any ongoing generation when interrupted
    - Emits LLM_RESPONSE_START/CHUNK/END events automatically
    - Is designed for voice agents, chatbots, and other interactive agents

    Usage:
        class MyVoiceAgent(OutputAgentNode):
            def __init__(self):
                super().__init__(name="voice-agent")
                self.llm = OpenAIClient(model="gpt-4o-mini")

            async def generate_response(self, messages):
                async for chunk in self.llm.chat(messages, stream=True):
                    if chunk.content:
                        yield chunk.content
    """

    def __init__(self, name: str, is_interruptible: bool = True):
        """
        Initialize output agent node.

        Args:
            name: Node name
        """
        super().__init__(name, is_interruptible)
        self.settings = OutputAgentSettings()
        self.context = ContextManager()

    async def start(self, init_event: SDKSystemInitEvent, task_manager: TaskManager):
        """Start the node"""
        await super().start(init_event, task_manager)

    async def _update_settings(self, settings: Dict[str, Any]):
        """Update the settings for this node"""
        for key, value in settings.items():
            logger.debug(f"[{self.name}] Updating setting {key} to {value}")
            setattr(self.settings, key, value)

    async def process_event(self, event: SDKEvent):
        """
        Route events to appropriate handlers.

        Handles:
        - LLMRequestEvent -> Start generation
        """

        await super().process_event(event)

        if isinstance(event, SDKSystemLLMRequestEvent):
            await self._handle_llm_request()
        elif isinstance(event, SDKAgentTranscriptUpdateEvent):
            self.context.add_message({"role": event.role, "content": event.content})
        elif isinstance(event, SDKSystemUpdateOutputAgentSettingsEvent):
            await self._update_settings(event.settings)

        await self.send_event(event)

    async def speak(self, text: str):
        """Send the given text"""

        if not self._running:
            logger.warning(f"[{self.name}] Not running, cannot send message")
            return

        await self.send_event(SDKAgentSpeakEvent(text=text))

    async def _handle_llm_request(self):
        """
        Handle LLM request by starting response generation.

        Cancels any existing generation task before starting new one.
        """
        try:
            await self.send_event(SDKAgentLLMResponseStartEvent())

            async for chunk in self.generate_response():
                chunk_event = SDKAgentLLMResponseChunkEvent(text=chunk)
                await self.send_event(chunk_event)

        except Exception as e:
            logger.exception(f"[{self.name}] Error during generation: {e}")
            error_event = SDKAgentErrorEvent(
                message=str(e),
            )
            await self.send_event(error_event)

        finally:
            await self.send_event(SDKAgentLLMResponseEndEvent())
            logger.debug(f"[{self.name}] Generation completed")

    @abstractmethod
    async def generate_response(self) -> AsyncIterator[str]:
        """
        Generate response chunks (implement in subclass).

        This is where you call your LLM client and yield text chunks.

        Args:
            messages: Conversation messages

        Yields:
            Text chunks to stream

        Example:
            async def generate_response(self, messages):
                async for chunk in self.llm.chat(messages, stream=True):
                    if chunk.content:
                        yield chunk.content
        """
        pass



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawSpeechToTextClient, RawSpeechToTextClient
from .types.lightning_speech_to_text_request_age_detection import LightningSpeechToTextRequestAgeDetection
from .types.lightning_speech_to_text_request_emotion_detection import LightningSpeechToTextRequestEmotionDetection
from .types.lightning_speech_to_text_request_gender_detection import LightningSpeechToTextRequestGenderDetection
from .types.lightning_speech_to_text_request_language import LightningSpeechToTextRequestLanguage
from .types.lightning_speech_to_text_request_model import LightningSpeechToTextRequestModel
from .types.lightning_speech_to_text_response import LightningSpeechToTextResponse
from .types.pulse_speech_to_text_request_age_detection import PulseSpeechToTextRequestAgeDetection
from .types.pulse_speech_to_text_request_emotion_detection import PulseSpeechToTextRequestEmotionDetection
from .types.pulse_speech_to_text_request_gender_detection import PulseSpeechToTextRequestGenderDetection
from .types.pulse_speech_to_text_request_language import PulseSpeechToTextRequestLanguage
from .types.pulse_speech_to_text_request_model import PulseSpeechToTextRequestModel
from .types.pulse_speech_to_text_response import PulseSpeechToTextResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SpeechToTextClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSpeechToTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSpeechToTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSpeechToTextClient
        """
        return self._raw_client

    def lightning(
        self,
        *,
        model: LightningSpeechToTextRequestModel,
        request: typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]],
        language: typing.Optional[LightningSpeechToTextRequestLanguage] = None,
        webhook_url: typing.Optional[str] = None,
        webhook_extra: typing.Optional[str] = None,
        word_timestamps: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        age_detection: typing.Optional[LightningSpeechToTextRequestAgeDetection] = None,
        gender_detection: typing.Optional[LightningSpeechToTextRequestGenderDetection] = None,
        emotion_detection: typing.Optional[LightningSpeechToTextRequestEmotionDetection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LightningSpeechToTextResponse:
        """
        Convert speech to text using the Lightning ASR model. Supports two input methods - raw audio bytes (application/octet-stream) with query parameters, or audio URL (application/json) with URL in body.

        Parameters
        ----------
        model : LightningSpeechToTextRequestModel
            The ASR model to use for transcription

        request : typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]]

        language : typing.Optional[LightningSpeechToTextRequestLanguage]
            Language of the audio file. Use `multi` for automatic language detection

        webhook_url : typing.Optional[str]

        webhook_extra : typing.Optional[str]

        word_timestamps : typing.Optional[bool]
            Whether to include word and utterance level timestamps in the response

        diarize : typing.Optional[bool]
            Whether to perform speaker diarization

        age_detection : typing.Optional[LightningSpeechToTextRequestAgeDetection]
            Whether to predict age group of the speaker

        gender_detection : typing.Optional[LightningSpeechToTextRequestGenderDetection]
            Whether to predict the gender of the speaker

        emotion_detection : typing.Optional[LightningSpeechToTextRequestEmotionDetection]
            Whether to predict speaker emotions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LightningSpeechToTextResponse
            Speech transcribed successfully
        """
        _response = self._raw_client.lightning(
            model=model,
            request=request,
            language=language,
            webhook_url=webhook_url,
            webhook_extra=webhook_extra,
            word_timestamps=word_timestamps,
            diarize=diarize,
            age_detection=age_detection,
            gender_detection=gender_detection,
            emotion_detection=emotion_detection,
            request_options=request_options,
        )
        return _response.data

    def pulse(
        self,
        *,
        model: PulseSpeechToTextRequestModel,
        request: typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]],
        language: typing.Optional[PulseSpeechToTextRequestLanguage] = None,
        webhook_url: typing.Optional[str] = None,
        webhook_extra: typing.Optional[str] = None,
        word_timestamps: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        age_detection: typing.Optional[PulseSpeechToTextRequestAgeDetection] = None,
        gender_detection: typing.Optional[PulseSpeechToTextRequestGenderDetection] = None,
        emotion_detection: typing.Optional[PulseSpeechToTextRequestEmotionDetection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PulseSpeechToTextResponse:
        """
        Convert speech to text using the Pulse ASR model. Supports two input methods - raw audio bytes (application/octet-stream) with query parameters, or audio URL (application/json) with URL in body.

        Parameters
        ----------
        model : PulseSpeechToTextRequestModel
            The ASR model to use for transcription

        request : typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]]

        language : typing.Optional[PulseSpeechToTextRequestLanguage]
            Language of the audio file. Use `multi` for automatic language detection

        webhook_url : typing.Optional[str]

        webhook_extra : typing.Optional[str]

        word_timestamps : typing.Optional[bool]
            Whether to include word and utterance level timestamps in the response

        diarize : typing.Optional[bool]
            Whether to perform speaker diarization

        age_detection : typing.Optional[PulseSpeechToTextRequestAgeDetection]
            Whether to predict age group of the speaker

        gender_detection : typing.Optional[PulseSpeechToTextRequestGenderDetection]
            Whether to predict the gender of the speaker

        emotion_detection : typing.Optional[PulseSpeechToTextRequestEmotionDetection]
            Whether to predict speaker emotions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PulseSpeechToTextResponse
            Speech transcribed successfully
        """
        _response = self._raw_client.pulse(
            model=model,
            request=request,
            language=language,
            webhook_url=webhook_url,
            webhook_extra=webhook_extra,
            word_timestamps=word_timestamps,
            diarize=diarize,
            age_detection=age_detection,
            gender_detection=gender_detection,
            emotion_detection=emotion_detection,
            request_options=request_options,
        )
        return _response.data


class AsyncSpeechToTextClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSpeechToTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSpeechToTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSpeechToTextClient
        """
        return self._raw_client

    async def lightning(
        self,
        *,
        model: LightningSpeechToTextRequestModel,
        request: typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]],
        language: typing.Optional[LightningSpeechToTextRequestLanguage] = None,
        webhook_url: typing.Optional[str] = None,
        webhook_extra: typing.Optional[str] = None,
        word_timestamps: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        age_detection: typing.Optional[LightningSpeechToTextRequestAgeDetection] = None,
        gender_detection: typing.Optional[LightningSpeechToTextRequestGenderDetection] = None,
        emotion_detection: typing.Optional[LightningSpeechToTextRequestEmotionDetection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LightningSpeechToTextResponse:
        """
        Convert speech to text using the Lightning ASR model. Supports two input methods - raw audio bytes (application/octet-stream) with query parameters, or audio URL (application/json) with URL in body.

        Parameters
        ----------
        model : LightningSpeechToTextRequestModel
            The ASR model to use for transcription

        request : typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]]

        language : typing.Optional[LightningSpeechToTextRequestLanguage]
            Language of the audio file. Use `multi` for automatic language detection

        webhook_url : typing.Optional[str]

        webhook_extra : typing.Optional[str]

        word_timestamps : typing.Optional[bool]
            Whether to include word and utterance level timestamps in the response

        diarize : typing.Optional[bool]
            Whether to perform speaker diarization

        age_detection : typing.Optional[LightningSpeechToTextRequestAgeDetection]
            Whether to predict age group of the speaker

        gender_detection : typing.Optional[LightningSpeechToTextRequestGenderDetection]
            Whether to predict the gender of the speaker

        emotion_detection : typing.Optional[LightningSpeechToTextRequestEmotionDetection]
            Whether to predict speaker emotions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LightningSpeechToTextResponse
            Speech transcribed successfully
        """
        _response = await self._raw_client.lightning(
            model=model,
            request=request,
            language=language,
            webhook_url=webhook_url,
            webhook_extra=webhook_extra,
            word_timestamps=word_timestamps,
            diarize=diarize,
            age_detection=age_detection,
            gender_detection=gender_detection,
            emotion_detection=emotion_detection,
            request_options=request_options,
        )
        return _response.data

    async def pulse(
        self,
        *,
        model: PulseSpeechToTextRequestModel,
        request: typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]],
        language: typing.Optional[PulseSpeechToTextRequestLanguage] = None,
        webhook_url: typing.Optional[str] = None,
        webhook_extra: typing.Optional[str] = None,
        word_timestamps: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        age_detection: typing.Optional[PulseSpeechToTextRequestAgeDetection] = None,
        gender_detection: typing.Optional[PulseSpeechToTextRequestGenderDetection] = None,
        emotion_detection: typing.Optional[PulseSpeechToTextRequestEmotionDetection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PulseSpeechToTextResponse:
        """
        Convert speech to text using the Pulse ASR model. Supports two input methods - raw audio bytes (application/octet-stream) with query parameters, or audio URL (application/json) with URL in body.

        Parameters
        ----------
        model : PulseSpeechToTextRequestModel
            The ASR model to use for transcription

        request : typing.Union[bytes, typing.Iterator[bytes], typing.AsyncIterator[bytes]]

        language : typing.Optional[PulseSpeechToTextRequestLanguage]
            Language of the audio file. Use `multi` for automatic language detection

        webhook_url : typing.Optional[str]

        webhook_extra : typing.Optional[str]

        word_timestamps : typing.Optional[bool]
            Whether to include word and utterance level timestamps in the response

        diarize : typing.Optional[bool]
            Whether to perform speaker diarization

        age_detection : typing.Optional[PulseSpeechToTextRequestAgeDetection]
            Whether to predict age group of the speaker

        gender_detection : typing.Optional[PulseSpeechToTextRequestGenderDetection]
            Whether to predict the gender of the speaker

        emotion_detection : typing.Optional[PulseSpeechToTextRequestEmotionDetection]
            Whether to predict speaker emotions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PulseSpeechToTextResponse
            Speech transcribed successfully
        """
        _response = await self._raw_client.pulse(
            model=model,
            request=request,
            language=language,
            webhook_url=webhook_url,
            webhook_extra=webhook_extra,
            word_timestamps=word_timestamps,
            diarize=diarize,
            age_detection=age_detection,
            gender_detection=gender_detection,
            emotion_detection=emotion_detection,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .get_audience_id_members_search_response_data_members_item import GetAudienceIdMembersSearchResponseDataMembersItem
from .get_audience_id_members_search_response_data_search_info import GetAudienceIdMembersSearchResponseDataSearchInfo


class GetAudienceIdMembersSearchResponseData(UncheckedBaseModel):
    members: typing.Optional[typing.List[GetAudienceIdMembersSearchResponseDataMembersItem]] = None
    search_info: typing_extensions.Annotated[
        typing.Optional[GetAudienceIdMembersSearchResponseDataSearchInfo],
        FieldMetadata(alias="searchInfo"),
        pydantic.Field(alias="searchInfo", description="Information about the search performed"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SinglePromptConfigToolsItemType = typing.Union[
    typing.Literal["end_call", "transfer_call", "api_call", "extract_dynamic_variables", "knowledge_base_search"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ..types.lightning_v31request_language import LightningV31RequestLanguage
from ..types.lightning_v31request_output_format import LightningV31RequestOutputFormat
from ..types.sample_rate import SampleRate
from .raw_client import AsyncRawTextToSpeechClient, RawTextToSpeechClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TextToSpeechClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTextToSpeechClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTextToSpeechClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTextToSpeechClient
        """
        return self._raw_client

    def synthesize_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Converts provided text to speech using the Lightning V3.1 model.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Synthesized speech retrieved successfully.

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.waves.text_to_speech.synthesize_lightning_v31speech(
            text="text",
            voice_id="voice_id",
        )
        """
        with self._raw_client.synthesize_lightning_v31speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            pronunciation_dicts=pronunciation_dicts,
            request_options=request_options,
        ) as r:
            yield from r.data

    def stream_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[typing.Any]:
        """
        Converts provided text to speech using the Lightning V3.1 model with streaming response.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[typing.Any]
            Synthesized speech retrieved successfully.

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        response = client.waves.text_to_speech.stream_lightning_v31speech(
            text="text",
            voice_id="voice_id",
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.stream_lightning_v31speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            pronunciation_dicts=pronunciation_dicts,
            request_options=request_options,
        ) as r:
            yield from r.data


class AsyncTextToSpeechClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTextToSpeechClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTextToSpeechClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTextToSpeechClient
        """
        return self._raw_client

    async def synthesize_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Converts provided text to speech using the Lightning V3.1 model.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Synthesized speech retrieved successfully.

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.waves.text_to_speech.synthesize_lightning_v31speech(
                text="text",
                voice_id="voice_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.synthesize_lightning_v31speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            pronunciation_dicts=pronunciation_dicts,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def stream_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[typing.Any]:
        """
        Converts provided text to speech using the Lightning V3.1 model with streaming response.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[typing.Any]
            Synthesized speech retrieved successfully.

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.waves.text_to_speech.stream_lightning_v31speech(
                text="text",
                voice_id="voice_id",
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.stream_lightning_v31speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            pronunciation_dicts=pronunciation_dicts,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class KnowledgeBaseItem(BaseModel):
    """
    KnowledgeBaseItem
    """ # noqa: E501
    id: StrictStr = Field(description="The unique identifier for the knowledge base item", alias="_id")
    item_type: StrictStr = Field(description="The type of the knowledge base item", alias="itemType")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Additional metadata for the item")
    knowledge_base_id: StrictStr = Field(description="The ID of the knowledge base this item belongs to", alias="knowledgeBaseId")
    processing_status: StrictStr = Field(description="The processing status of the item", alias="processingStatus")
    file_name: Optional[StrictStr] = Field(default=None, description="The name of the file (for file type items)", alias="fileName")
    content_type: Optional[StrictStr] = Field(default=None, description="The MIME type of the content", alias="contentType")
    size: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The size of the file in bytes")
    key: Optional[StrictStr] = Field(default=None, description="The storage key for the file")
    title: Optional[StrictStr] = Field(default=None, description="The title of the item")
    content: Optional[StrictStr] = Field(default=None, description="The content of the item (for text type items)")
    created_at: datetime = Field(description="The date and time when the item was created", alias="createdAt")
    updated_at: datetime = Field(description="The date and time when the item was last updated", alias="updatedAt")
    __properties: ClassVar[List[str]] = ["_id", "itemType", "metadata", "knowledgeBaseId", "processingStatus", "fileName", "contentType", "size", "key", "title", "content", "createdAt", "updatedAt"]

    @field_validator('item_type')
    def item_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['file', 'text']):
            raise ValueError("must be one of enum values ('file', 'text')")
        return value

    @field_validator('processing_status')
    def processing_status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['pending', 'processing', 'completed', 'failed']):
            raise ValueError("must be one of enum values ('pending', 'processing', 'completed', 'failed')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of KnowledgeBaseItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of KnowledgeBaseItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_id": obj.get("_id"),
            "itemType": obj.get("itemType"),
            "metadata": obj.get("metadata"),
            "knowledgeBaseId": obj.get("knowledgeBaseId"),
            "processingStatus": obj.get("processingStatus"),
            "fileName": obj.get("fileName"),
            "contentType": obj.get("contentType"),
            "size": obj.get("size"),
            "key": obj.get("key"),
            "title": obj.get("title"),
            "content": obj.get("content"),
            "createdAt": obj.get("createdAt"),
            "updatedAt": obj.get("updatedAt")
        })
        return _obj





import json
import base64
import time
import threading
import queue
from typing import Generator
from dataclasses import dataclass
from websocket import WebSocketApp

@dataclass
class TTSConfig:
    voice_id: str
    api_key: str
    language: str = "en"
    sample_rate: int = 24000
    speed: float = 1.0
    consistency: float = 0.5
    enhancement: int = 1
    similarity: float = 0
    max_buffer_flush_ms: int = 0

class WavesStreamingTTS:
    def __init__(self, config: TTSConfig):
        self.config = config
        self.ws_url = "wss://api.smallest.ai/waves/v1/lightning-v2/get_speech/stream"
        self.ws = None
        self.audio_queue = queue.Queue()
        self.error_queue = queue.Queue()
        self.is_complete = False
        self.is_connected = False
        self.request_id = None

    def _get_headers(self):
        return [f"Authorization: Bearer {self.config.api_key}"]

    def _create_payload(self, text: str, continue_stream: bool = False, flush: bool = False):
        return {
            "voice_id": self.config.voice_id,
            "text": text,
            "language": self.config.language,
            "sample_rate": self.config.sample_rate,
            "speed": self.config.speed,
            "consistency": self.config.consistency,
            "similarity": self.config.similarity,
            "enhancement": self.config.enhancement,
            "max_buffer_flush_ms": self.config.max_buffer_flush_ms,
            "continue": continue_stream,
            "flush": flush
        }

    def _on_open(self, ws):
        self.is_connected = True

    def _on_message(self, ws, message):
        try:
            data = json.loads(message)
            status = data.get("status", "")

            if status == "error":
                self.error_queue.put(Exception(data.get("message", "Unknown error")))
                return

            if not self.request_id:
                self.request_id = data.get("request_id")

            audio_b64 = data.get("data", {}).get("audio")
            if audio_b64:
                self.audio_queue.put(base64.b64decode(audio_b64))

            if status == "complete":
                self.is_complete = True
                self.audio_queue.put(None)

        except Exception as e:
            self.error_queue.put(e)

    def _on_error(self, ws, error):
        self.error_queue.put(error)

    def _on_close(self, ws, *args):
        self.is_connected = False
        if not self.is_complete:
            self.audio_queue.put(None)

    def _connect(self):
        if self.ws:
            self.ws.close()

        self.ws = WebSocketApp(
            self.ws_url,
            header=self._get_headers(),
            on_open=self._on_open,
            on_message=self._on_message,
            on_error=self._on_error,
            on_close=self._on_close
        )

        ws_thread = threading.Thread(target=self.ws.run_forever)
        ws_thread.daemon = True
        ws_thread.start()

        timeout = 5.0
        start_time = time.time()
        while not self.is_connected and time.time() - start_time < timeout:
            time.sleep(0.1)

        if not self.is_connected:
            raise Exception("Failed to connect to WebSocket")

    def synthesize(self, text: str) -> Generator[bytes, None, None]:
        self._reset_state()
        self._connect()

        payload = self._create_payload(text)
        self.ws.send(json.dumps(payload))

        while True:
            if not self.error_queue.empty():
                raise self.error_queue.get()

            try:
                chunk = self.audio_queue.get(timeout=1.0)
                if chunk is None:
                    break
                yield chunk
            except queue.Empty:
                if self.is_complete:
                    break
                continue

        self.ws.close()

    def synthesize_streaming(self, text_stream: Generator[str, None, None],
                           continue_stream: bool = True,
                           auto_flush: bool = True) -> Generator[bytes, None, None]:
        self._reset_state()
        self._connect()

        def send_text():
            try:
                for text_chunk in text_stream:
                    if text_chunk.strip():
                        payload = self._create_payload(text_chunk, continue_stream=continue_stream)
                        self.ws.send(json.dumps(payload))

                if auto_flush:
                    flush_payload = self._create_payload("", flush=True)
                    self.ws.send(json.dumps(flush_payload))
            except Exception as e:
                self.error_queue.put(e)

        sender_thread = threading.Thread(target=send_text)
        sender_thread.daemon = True
        sender_thread.start()

        while True:
            if not self.error_queue.empty():
                raise self.error_queue.get()

            try:
                chunk = self.audio_queue.get(timeout=1.0)
                if chunk is None:
                    break
                yield chunk
            except queue.Empty:
                if self.is_complete:
                    break
                continue

        self.ws.close()

    def send_text_chunk(self, text: str, continue_stream: bool = True, flush: bool = False):
        if not self.is_connected:
            raise Exception("WebSocket not connected")
        payload = self._create_payload(text, continue_stream=continue_stream, flush=flush)
        self.ws.send(json.dumps(payload))

    def flush_buffer(self):
        if not self.is_connected:
            raise Exception("WebSocket not connected")
        payload = self._create_payload("", flush=True)
        self.ws.send(json.dumps(payload))

    def start_streaming_session(self) -> Generator[bytes, None, None]:
        self._reset_state()
        self._connect()

        while True:
            if not self.error_queue.empty():
                raise self.error_queue.get()

            try:
                chunk = self.audio_queue.get(timeout=0.1)
                if chunk is None:
                    break
                yield chunk
            except queue.Empty:
                if self.is_complete:
                    break
                continue

    def _reset_state(self):
        self.audio_queue = queue.Queue()
        self.error_queue = queue.Queue()
        self.is_complete = False
        self.is_connected = False
        self.request_id = None


# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class GetClonedVoicesResponseVoicesItem(UncheckedBaseModel):
    display_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="displayName"),
        pydantic.Field(alias="displayName", description="Display name for the voice."),
    ]
    accent: typing.Optional[str] = pydantic.Field(default=None)
    """
    Accent of the voice.
    """

    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of tags associated with the voice.
    """

    voice_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="voiceId"), pydantic.Field(alias="voiceId", description="Unique Voice ID.")
    ]
    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    Model used to generate the voice.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status of the voice generation.
    """

    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="Date and time the voice was created."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .patch_agent_id_request_synthesizer_voice_config_one_model import PatchAgentIdRequestSynthesizerVoiceConfigOneModel


class PatchAgentIdRequestSynthesizerVoiceConfigOne(UncheckedBaseModel):
    model: typing.Optional[PatchAgentIdRequestSynthesizerVoiceConfigOneModel] = pydantic.Field(default=None)
    """
    We currently support 3 types of models for the synthesizer. Waves, Waves Lightning Large and Waves Lightning Large Voice Clone. You can clone your voice using waves platform and use the voiceId for this field and select the model as waves_lightning_large_voice_clone to use your cloned voice.
    """

    voice_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="voiceId"),
        pydantic.Field(alias="voiceId", description="The voice ID to use"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from .agent_dto_background_sound import AgentDtoBackgroundSound
from .agent_dto_language import AgentDtoLanguage
from .agent_dto_slm_model import AgentDtoSlmModel
from .agent_dto_synthesizer import AgentDtoSynthesizer
from .workflow_type import WorkflowType


class AgentDto(UncheckedBaseModel):
    id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="_id"), pydantic.Field(alias="_id", description="The ID of the agent")
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the agent
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the agent
    """

    background_sound: typing_extensions.Annotated[
        typing.Optional[AgentDtoBackgroundSound],
        FieldMetadata(alias="backgroundSound"),
        pydantic.Field(
            alias="backgroundSound",
            description="Ambient background sound during calls. Options: '' (none), 'office', 'cafe', 'call_center', 'static'.",
        ),
    ] = None
    organization: typing.Optional[str] = pydantic.Field(default=None)
    """
    The organization ID of the agent
    """

    workflow_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="workflowId"),
        pydantic.Field(alias="workflowId", description="The workflow ID of the agent"),
    ] = None
    workflow_type: typing_extensions.Annotated[
        typing.Optional[WorkflowType],
        FieldMetadata(alias="workflowType"),
        pydantic.Field(alias="workflowType", description="The type of workflow used by the agent"),
    ] = None
    created_by: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdBy"),
        pydantic.Field(alias="createdBy", description="The user ID of the user who created the agent"),
    ] = None
    global_knowledge_base_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="globalKnowledgeBaseId"),
        pydantic.Field(alias="globalKnowledgeBaseId", description="The global knowledge base ID of the agent"),
    ] = None
    language: typing.Optional[AgentDtoLanguage] = pydantic.Field(default=None)
    """
    The language configuration of the agent
    """

    synthesizer: typing.Optional[AgentDtoSynthesizer] = pydantic.Field(default=None)
    """
    The synthesizer (TTS) configuration of the agent
    """

    slm_model: typing_extensions.Annotated[
        typing.Optional[AgentDtoSlmModel],
        FieldMetadata(alias="slmModel"),
        pydantic.Field(
            alias="slmModel",
            description="The LLM model to use for the agent. LLM model will be used to generate the response and take decisions based on the user's query.",
        ),
    ] = None
    default_variables: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]],
        FieldMetadata(alias="defaultVariables"),
        pydantic.Field(
            alias="defaultVariables",
            description="The default variables to use for the agent. These variables will be used if no variables are provided when initiating a conversation with the agent.",
        ),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The date and time when the agent was created"),
    ] = None
    updated_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The date and time when the agent was last updated"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class GetConversationResponseDataPagination(UncheckedBaseModel):
    total: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of matching call logs
    """

    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    Current page number
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of items per page (page size)
    """

    has_more: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasMore"),
        pydantic.Field(alias="hasMore", description="Whether there are more pages available"),
    ] = None
    total_pages: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalPages"),
        pydantic.Field(alias="totalPages", description="Total number of pages"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from smallestai.atoms.models.agent_from_template_post200_response import AgentFromTemplatePost200Response
from smallestai.atoms.models.agent_template_get200_response import AgentTemplateGet200Response
from smallestai.atoms.models.create_agent_from_template_request import CreateAgentFromTemplateRequest

from smallestai.atoms.api_client import ApiClient, RequestSerialized
from smallestai.atoms.api_response import ApiResponse
from smallestai.atoms.rest import RESTResponseType


class AgentTemplatesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def agent_from_template_post(
        self,
        create_agent_from_template_request: CreateAgentFromTemplateRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentFromTemplatePost200Response:
        """Create agent from template

        We have created templates for some common use cases. You can use these templates to create an agent. For getting list of templates, you can use the /agent/template endpoint. It will give you the list of templates with their description and id. You can pass the id of the template in the request body to create an agent from the template.

        :param create_agent_from_template_request: (required)
        :type create_agent_from_template_request: CreateAgentFromTemplateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_from_template_post_serialize(
            create_agent_from_template_request=create_agent_from_template_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentFromTemplatePost200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def agent_from_template_post_with_http_info(
        self,
        create_agent_from_template_request: CreateAgentFromTemplateRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentFromTemplatePost200Response]:
        """Create agent from template

        We have created templates for some common use cases. You can use these templates to create an agent. For getting list of templates, you can use the /agent/template endpoint. It will give you the list of templates with their description and id. You can pass the id of the template in the request body to create an agent from the template.

        :param create_agent_from_template_request: (required)
        :type create_agent_from_template_request: CreateAgentFromTemplateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_from_template_post_serialize(
            create_agent_from_template_request=create_agent_from_template_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentFromTemplatePost200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def agent_from_template_post_without_preload_content(
        self,
        create_agent_from_template_request: CreateAgentFromTemplateRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create agent from template

        We have created templates for some common use cases. You can use these templates to create an agent. For getting list of templates, you can use the /agent/template endpoint. It will give you the list of templates with their description and id. You can pass the id of the template in the request body to create an agent from the template.

        :param create_agent_from_template_request: (required)
        :type create_agent_from_template_request: CreateAgentFromTemplateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_from_template_post_serialize(
            create_agent_from_template_request=create_agent_from_template_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentFromTemplatePost200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _agent_from_template_post_serialize(
        self,
        create_agent_from_template_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_agent_from_template_request is not None:
            _body_params = create_agent_from_template_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/agent/from-template',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def agent_template_get(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentTemplateGet200Response:
        """Get agent templates


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_template_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentTemplateGet200Response",
            '401': "UnauthorizedErrorReponse",
            '400': "BadRequestErrorResponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def agent_template_get_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentTemplateGet200Response]:
        """Get agent templates


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_template_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentTemplateGet200Response",
            '401': "UnauthorizedErrorReponse",
            '400': "BadRequestErrorResponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def agent_template_get_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get agent templates


        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._agent_template_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentTemplateGet200Response",
            '401': "UnauthorizedErrorReponse",
            '400': "BadRequestErrorResponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _agent_template_get_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/agent/template',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





"""
Campaign management.

Usage:
    from smallestai.atoms.helpers import Campaign

    campaign = Campaign()
    campaign.create(name="My Campaign", agent_id="...", audience_id="...", phone_ids=["..."])
    campaign.start(campaign_id)
"""

import os
from typing import Any, Dict, List, Optional

import requests

# Default API base URL
DEFAULT_BASE_URL = "https://api.smallest.ai/atoms/v1"


class Campaign:
    """
    Manager for campaign operations.

    Can be used standalone:
        campaign = Campaign()
        campaign.create(...)
    """

    def __init__(
        self,
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
    ):
        """
        Initialize Campaign manager.

        Args:
            base_url: API base URL (default: api.smallest.ai)
            api_key: API key (default: SMALLEST_API_KEY env var)
        """
        self.base_url = base_url or os.environ.get("SMALLEST_BASE_URL", DEFAULT_BASE_URL)
        self.api_key = api_key or os.environ.get("SMALLEST_API_KEY", "")

    def _get_headers(self) -> Dict[str, str]:
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

    # =========================================================================
    # CRUD Operations
    # =========================================================================

    def create(
        self,
        name: str,
        agent_id: str,
        audience_id: str,
        phone_ids: List[str],
        description: str = "",
        max_retries: int = 3,
        retry_delay: int = 15,
    ) -> Dict[str, Any]:
        """Create a new campaign."""
        url = f"{self.base_url}/campaign"
        payload = {
            "name": name,
            "agentId": agent_id,
            "audienceId": audience_id,
            "phoneNumberIds": phone_ids,
            "description": description,
            "maxRetries": max_retries,
            "retryDelay": retry_delay,
        }
        response = requests.post(url, headers=self._get_headers(), json=payload)
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def get(self, campaign_id: str) -> Dict[str, Any]:
        """Get campaign details."""
        url = f"{self.base_url}/campaign/{campaign_id}"
        response = requests.get(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def list(self, limit: int = 50) -> Dict[str, Any]:
        """List all campaigns."""
        url = f"{self.base_url}/campaign"
        params = {"limit": limit}
        response = requests.get(url, headers=self._get_headers(), params=params)
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def delete(self, campaign_id: str) -> Dict[str, Any]:
        """Delete a campaign."""
        url = f"{self.base_url}/campaign/{campaign_id}"
        response = requests.delete(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    # =========================================================================
    # Campaign Control
    # =========================================================================

    def start(self, campaign_id: str) -> Dict[str, Any]:
        """Start a campaign."""
        url = f"{self.base_url}/campaign/{campaign_id}/start"
        response = requests.post(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def stop(self, campaign_id: str) -> Dict[str, Any]:
        """Stop a campaign."""
        url = f"{self.base_url}/campaign/{campaign_id}/stop"
        response = requests.post(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def pause(self, campaign_id: str) -> Dict[str, Any]:
        """Pause a campaign."""
        url = f"{self.base_url}/campaign/{campaign_id}/pause"
        response = requests.post(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]



# This file was auto-generated by Fern from our API Definition.

import typing

CreateAgentRequestBackgroundSound = typing.Union[
    typing.Literal["", "office", "cafe", "call_center", "static"], typing.Any
]



import fnmatch
import io
import zipfile
from pathlib import Path

from loguru import logger
from rich.console import Console

console = Console()


def create_zip_from_directory(directory: Path):
    """Create a zip file from a directory, excluding files based on .gitignore patterns."""

    excluded_patterns = {
        # Byte-compiled / optimized / DLL files
        "__pycache__",
        "*.py[codz]",
        "*$py.class",
        "*.so",
        # Distribution / packaging
        ".Python",
        "build",
        "develop-eggs",
        "dist",
        "downloads",
        "eggs",
        ".eggs",
        "lib",
        "lib64",
        "parts",
        "sdist",
        "var",
        "wheels",
        "share",
        "*.egg-info",
        ".installed.cfg",
        "*.egg",
        "MANIFEST",
        # PyInstaller
        "*.manifest",
        "*.spec",
        # Installer logs
        "pip-log.txt",
        "pip-delete-this-directory.txt",
        # Unit test / coverage reports
        "htmlcov",
        ".tox",
        ".nox",
        ".coverage",
        ".coverage.*",
        ".cache",
        "nosetests.xml",
        "coverage.xml",
        "*.cover",
        "*.py.cover",
        ".hypothesis",
        ".pytest_cache",
        "cover",
        # Translations
        "*.mo",
        "*.pot",
        # Django
        "*.log",
        "local_settings.py",
        "db.sqlite3",
        "db.sqlite3-journal",
        # Flask
        "instance",
        ".webassets-cache",
        # Scrapy
        ".scrapy",
        # Sphinx documentation
        "docs/_build",
        # PyBuilder
        ".pybuilder",
        "target",
        # Jupyter Notebook
        ".ipynb_checkpoints",
        # IPython
        "profile_default",
        "ipython_config.py",
        # pipenv, UV, poetry, pdm, pixi
        ".venv",
        "env",
        "venv",
        "ENV",
        "env.bak",
        "venv.bak",
        ".pdm-python",
        ".pdm-build",
        ".pixi",
        # PEP 582
        "__pypackages__",
        # Celery
        "celerybeat-schedule",
        "celerybeat.pid",
        # Redis
        "*.rdb",
        "*.aof",
        "*.pid",
        # RabbitMQ
        "mnesia",
        "rabbitmq",
        "rabbitmq-data",
        # ActiveMQ
        "activemq-data",
        # SageMath
        "*.sage.py",
        # Environments
        #
        # Disabling env for now until environment variable feature is not implemented on platform
        # ".env",
        ".envrc",
        # IDE settings
        ".spyderproject",
        ".spyproject",
        ".ropeproject",
        # mkdocs
        "site",
        # Type checkers
        ".mypy_cache",
        ".dmypy.json",
        "dmypy.json",
        ".pyre",
        ".pytype",
        # Cython
        "cython_debug",
        # Abstra
        ".abstra",
        # Ruff
        ".ruff_cache",
        # PyPI
        ".pypirc",
        # Marimo
        "marimo/_static",
        "marimo/_lsp",
        "__marimo__",
        # Streamlit
        ".streamlit",
        # Git
        ".git",
        # Node
        "node_modules",
    }

    def should_exclude(path: Path, relative_path: Path) -> bool:
        """Check if a path should be excluded based on patterns."""
        path_str = str(relative_path)
        path_parts = relative_path.parts

        for part in path_parts:
            for pattern in excluded_patterns:
                if part == pattern:
                    return True

                if fnmatch.fnmatch(part, pattern):
                    return True

        for pattern in excluded_patterns:
            if fnmatch.fnmatch(path_str, pattern) or fnmatch.fnmatch(
                path_str, f"*/{pattern}"
            ):
                return True

            if fnmatch.fnmatch(path.name, pattern):
                return True

        return False

    zip_buffer = io.BytesIO()
    with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zip_file:
        for file_path in directory.rglob("*"):
            if file_path.is_file():
                relative_path = file_path.relative_to(directory)

                if should_exclude(file_path, relative_path):
                    continue

                console.print(f"[dim]Adding file: {relative_path}[/dim]")

                zip_file.write(file_path, relative_path)

    zip_buffer.seek(0)
    return zip_buffer



import os
import json
import copy
import requests
from typing import Optional, Union, List

from smallestai.waves.exceptions import InvalidError, APIError
from smallestai.waves.utils import (TTSOptions, validate_input, validate_asr_input,
                        get_smallest_languages, get_smallest_models, ALLOWED_AUDIO_EXTENSIONS, API_BASE_URL)

class WavesClient:
    def __init__(
        self,
        api_key: str = None,
        model: Optional[str] = "lightning",
        sample_rate: Optional[int] = 24000,
        voice_id: Optional[str] = "emily",
        speed: Optional[float] = 1.0,
        consistency: Optional[float] = 0.5,
        similarity: Optional[float] = 0.0,
        enhancement: Optional[int] = 1,
        language: Optional[str] = "en",
        output_format: Optional[str] = "wav"
    ) -> None:
        """
        Smallest Instance for text-to-speech synthesis.

        This is a synchronous implementation of the text-to-speech functionality.
        For an asynchronous version, please refer to the AsyncSmallest Instance.

        Args:
        - api_key (str): The API key for authentication, export it as 'SMALLEST_API_KEY' in your environment variables.
        - model (TTSModels): The model to be used for synthesis.
        - sample_rate (int): The sample rate for the audio output.
        - voice_id (TTSVoices): The voice to be used for synthesis.
        - speed (float): The speed of the speech synthesis.
        - consistency (float): This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition. Only supported in `lightning-large` model. Range - [0, 1]
        - similarity (float): This parameter controls the similarity between the synthesized audio and the reference audio. Increase it to make the speech more similar to the reference audio. Only supported in `lightning-large` model. Range - [0, 1]
        - enhancement (int): Enhances speech quality at the cost of increased latency. Only supported in `lightning-large` model. Range - [0, 2].
        - language (str): The language for synthesis. Default is "en".
        - output_format (str): The output audio format. Options: "pcm", "mp3", "wav", "mulaw". Default is "pcm".

        Methods:
        - get_languages: Returns a list of available languages for synthesis.
        - get_voices: Returns a list of available voices for synthesis.
        - get_models: Returns a list of available models for synthesis.
        - synthesize: Converts the provided text into speech and returns the audio content.
        """
        self.api_key = api_key or os.environ.get("SMALLEST_API_KEY")
        if not self.api_key:
            raise InvalidError()
        if model == "lightning-large" and voice_id is None:
            voice_id = "lakshya"

        self.chunk_size = 250

        self.opts = TTSOptions(
            model=model,
            sample_rate=sample_rate,
            voice_id=voice_id,
            api_key=self.api_key,
            speed=speed,
            consistency=consistency,
            similarity=similarity,
            enhancement=enhancement,
            language=language,
            output_format=output_format
        )


    def get_languages(self, model:str="lightning") -> List[str]:
        """Returns a list of available languages."""
        return get_smallest_languages(model)

    def get_cloned_voices(self) -> str:
        """Returns a list of your cloned voices."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
        }

        res = requests.request("GET", f"{API_BASE_URL}/lightning-large/get_cloned_voices", headers=headers)
        if res.status_code != 200:
            raise APIError(f"Failed to get cloned voices: {res.text}. For more information, visit https://waves.smallest.ai/")

        return json.dumps(res.json(), indent=4, ensure_ascii=False)


    def get_voices(
            self,
            model: Optional[str] = "lightning"
        ) -> str:
        """Returns a list of available voices."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
        }

        res = requests.request("GET", f"{API_BASE_URL}/{model}/get_voices", headers=headers)
        if res.status_code != 200:
            raise APIError(f"Failed to get voices: {res.text}. For more information, visit https://waves.smallest.ai/")

        return json.dumps(res.json(), indent=4, ensure_ascii=False)


    def get_models(self) -> List[str]:
        """Returns a list of available models."""
        return get_smallest_models()


    def synthesize(
            self,
            text: str,
            **kwargs
        ) -> Union[bytes]:
        """
        Synthesize speech from the provided text.

        - text (str): The text to be converted to speech.
        - stream (Optional[bool]): If True, returns an iterator yielding audio chunks instead of a full byte array.
        - kwargs: Additional optional parameters to override `__init__` options for this call.

        Returns:
        - Union[bytes, None, Iterator[bytes]]:
            - If `stream=True`, returns an iterator yielding audio chunks.
            - If `save_as` is provided, saves the file and returns None.
            - Otherwise, returns the synthesized audio content as bytes.

        Raises:
        - InvalidError: If the provided file name does not have a .wav or .mp3 extension when `save_as` is specified.
        - APIError: If the API request fails or returns an error.
        """
        opts = copy.deepcopy(self.opts)
        valid_keys = set(vars(opts).keys())

        invalid_keys = [key for key in kwargs if key not in valid_keys]
        if invalid_keys:
            raise ValueError(f"Invalid parameter(s) in kwargs: {', '.join(invalid_keys)}. Allowed parameters are: {', '.join(valid_keys)}")

        for key, value in kwargs.items():
            setattr(opts, key, value)

        validate_input(text, opts.model, opts.sample_rate, opts.speed, opts.consistency, opts.similarity, opts.enhancement)

        payload = {
            "text": text,
            "voice_id": opts.voice_id,
            "sample_rate": opts.sample_rate,
            "speed": opts.speed,
            "consistency": opts.consistency,
            "similarity": opts.similarity,
            "enhancement": opts.enhancement,
            "language": opts.language,
            "output_format": opts.output_format
        }

        if opts.model == "lightning-large" or opts.model == "lightning-v2":
            if opts.consistency is not None:
                payload["consistency"] = opts.consistency
            if opts.similarity is not None:
                payload["similarity"] = opts.similarity
            if opts.enhancement is not None:
                payload["enhancement"] = opts.enhancement

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        res = requests.post(f"{API_BASE_URL}/{opts.model}/get_speech", json=payload, headers=headers)
        if res.status_code != 200:
            raise APIError(f"Failed to synthesize speech: {res.text}. Please check if you have set the correct API key. For more information, visit https://waves.smallest.ai/")

        return res.content


    def add_voice(self, display_name: str, file_path: str) -> str:
        """
        Instantly clone your voice synchronously.

        Args:
        - display_name (str): The display name for the new voice.
        - file_path (str): The path to the reference audio file to be cloned.

        Returns:
        - str: The response from the API as a formatted JSON string.

        Raises:
        - InvalidError: If the file does not exist or is not a valid audio file.
        - APIError: If the API request fails or returns an error.
        """
        if not os.path.isfile(file_path):
            raise InvalidError("Invalid file path. File does not exist.")

        file_extension = os.path.splitext(file_path)[1].lower()
        if file_extension not in ALLOWED_AUDIO_EXTENSIONS:
            raise InvalidError(f"Invalid file type. Supported formats are: {ALLOWED_AUDIO_EXTENSIONS}")

        url = f"{API_BASE_URL}/lightning-large/add_voice"
        payload = {'displayName': display_name}

        files = [('file', (os.path.basename(file_path), open(file_path, 'rb'), 'audio/wav'))]

        headers = {
            'Authorization': f"Bearer {self.api_key}",
        }

        response = requests.post(url, headers=headers, data=payload, files=files)
        if response.status_code != 200:
            raise APIError(f"Failed to add voice: {response.text}. For more information, visit https://waves.smallest.ai/")

        return response.json()


    def delete_voice(self, voice_id: str) -> str:
        """
        Delete a cloned voice synchronously.

        Args:
        - voice_id (str): The ID of the voice to be deleted.

        Returns:
        - str: The response from the API.

        Raises:
        - APIError: If the API request fails or returns an error.
        """
        url = f"{API_BASE_URL}/lightning-large"
        payload = {'voiceId': voice_id}

        headers = {
            'Authorization': f"Bearer {self.api_key}",
        }

        response = requests.delete(url, headers=headers, json=payload)
        if response.status_code != 200:
            raise APIError(f"Failed to delete voice: {response.text}. For more information, visit https://waves.smallest.ai/")

        return response.json()

    def transcribe(
        self,
        file_path: str,
        language: Optional[str] = "en",
        word_timestamps: Optional[bool] = False,
        age_detection: Optional[bool] = False,
        gender_detection: Optional[bool] = False,
        emotion_detection: Optional[bool] = False,
        model: Optional[str] = "lightning"
    ) -> dict:
        validate_asr_input(file_path, model, language)

        url = f"{API_BASE_URL}/speech-to-text"
        headers = {
            'Authorization': f"Bearer {self.api_key}",
        }
        payload = {
            'model': model,
            'language': language,
            'word_timestamps': str(bool(word_timestamps)).lower(),
            'age_detection': str(bool(age_detection)).lower(),
            'gender_detection': str(bool(gender_detection)).lower(),
            'emotion_detection': str(bool(emotion_detection)).lower()
        }

        file_extension = os.path.splitext(file_path)[1].lower()
        content_type = f"audio/{file_extension[1:]}" if file_extension else "application/octet-stream"

        with open(file_path, 'rb') as f:
            files = {'file': (os.path.basename(file_path), f, content_type)}
            response = requests.post(url, headers=headers, files=files, data=payload)

        if response.status_code != 200:
            raise APIError(f"Failed to transcribe audio: {response.text}. For more information, visit https://waves-docs.smallest.ai/v4.0.0/content/api-references/asr-post-api")

        return response.json()


# This file was auto-generated by Fern from our API Definition.

import typing

from .patch_agent_id_request_synthesizer_voice_config_gender import PatchAgentIdRequestSynthesizerVoiceConfigGender
from .patch_agent_id_request_synthesizer_voice_config_one import PatchAgentIdRequestSynthesizerVoiceConfigOne

PatchAgentIdRequestSynthesizerVoiceConfig = typing.Union[
    PatchAgentIdRequestSynthesizerVoiceConfigGender, PatchAgentIdRequestSynthesizerVoiceConfigOne
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class LightningSpeechToTextResponseMetadata(UncheckedBaseModel):
    """
    Metadata about the transcription
    """

    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the audio file
    """

    duration: typing.Optional[float] = pydantic.Field(default=None)
    """
    Duration of the audio file in minutes
    """

    file_size: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="fileSize"),
        pydantic.Field(alias="fileSize", description="Size of the audio file in bytes"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel


class WebhookEvent(UncheckedBaseModel):
    id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="_id"),
        pydantic.Field(alias="_id", description="The unique identifier for the event"),
    ] = None
    webhook_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="webhookId"),
        pydantic.Field(alias="webhookId", description="The ID of the webhook"),
    ] = None
    event_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="eventType"),
        pydantic.Field(alias="eventType", description="The type of event"),
    ] = None
    payload: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    The event payload
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the event
    """

    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The date and time when the event was created"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class AgentIdPatchRequestLanguageSwitching(BaseModel):
    """
    Language switching configuration for the agent. If enabled, the agent will be able to switch between languages based on the user's language.
    """ # noqa: E501
    is_enabled: Optional[StrictBool] = Field(default=False, description="Whether to enable language switching for the agent", alias="isEnabled")
    min_words_for_detection: Optional[Union[Annotated[float, Field(le=10, strict=True, ge=1)], Annotated[int, Field(le=10, strict=True, ge=1)]]] = Field(default=2, description="Minimum number of words required for language detection", alias="minWordsForDetection")
    strong_signal_threshold: Optional[Union[Annotated[float, Field(le=0.9, strict=True, ge=0.1)], Annotated[int, Field(le=0, strict=True, ge=1)]]] = Field(default=0.7, description="Threshold for strong language signal detection (0.1 to 0.9)", alias="strongSignalThreshold")
    weak_signal_threshold: Optional[Union[Annotated[float, Field(le=0.9, strict=True, ge=0.1)], Annotated[int, Field(le=0, strict=True, ge=1)]]] = Field(default=0.3, description="Threshold for weak language signal detection (0.1 to 0.9)", alias="weakSignalThreshold")
    min_consecutive_for_weak_threshold_switch: Optional[Union[Annotated[float, Field(le=5, strict=True, ge=1)], Annotated[int, Field(le=5, strict=True, ge=1)]]] = Field(default=2, description="Minimum consecutive detections required for weak threshold language switch", alias="minConsecutiveForWeakThresholdSwitch")
    __properties: ClassVar[List[str]] = ["isEnabled", "minWordsForDetection", "strongSignalThreshold", "weakSignalThreshold", "minConsecutiveForWeakThresholdSwitch"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AgentIdPatchRequestLanguageSwitching from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AgentIdPatchRequestLanguageSwitching from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "isEnabled": obj.get("isEnabled") if obj.get("isEnabled") is not None else False,
            "minWordsForDetection": obj.get("minWordsForDetection") if obj.get("minWordsForDetection") is not None else 2,
            "strongSignalThreshold": obj.get("strongSignalThreshold") if obj.get("strongSignalThreshold") is not None else 0.7,
            "weakSignalThreshold": obj.get("weakSignalThreshold") if obj.get("weakSignalThreshold") is not None else 0.3,
            "minConsecutiveForWeakThresholdSwitch": obj.get("minConsecutiveForWeakThresholdSwitch") if obj.get("minConsecutiveForWeakThresholdSwitch") is not None else 2
        })
        return _obj





# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .get_audience_id_members_response_data_members_item import GetAudienceIdMembersResponseDataMembersItem


class GetAudienceIdMembersResponseData(UncheckedBaseModel):
    members: typing.Optional[typing.List[GetAudienceIdMembersResponseDataMembersItem]] = None
    total_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalCount"),
        pydantic.Field(alias="totalCount", description="Total number of members in the audience"),
    ] = None
    total_pages: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalPages"),
        pydantic.Field(alias="totalPages", description="Total number of pages available"),
    ] = None
    has_more: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasMore"),
        pydantic.Field(alias="hasMore", description="Whether there are more pages available"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
